#include "testApp.h"


//--------------------------------------------------------------
void testApp::setup(){	
	
	ofSetVerticalSync(true);
	ofSetFrameRate(60);
	
	
	for (int i = 0; i < 800; i++){
		particle myParticle;
		myParticle.setInitialCondition(ofRandom(400,600),ofRandom(0,ofGetHeight()),0,0);
		particles.push_back(myParticle);
	}
	
	VF.setupField(60,40,ofGetWidth(), ofGetHeight());
	VF.randomizeField(2.0);
	bParticlesRepel = true;
	bFade = false;
}

//--------------------------------------------------------------
void testApp::update(){

	// on every frame 
	// we reset the forces
	// add in any forces on the particle
	// perfom damping and
	// then update
	for (int i = 0; i < particles.size(); i++){
		avgParticle.x += particles[i].pos.x;
		avgParticle.y += particles[i].pos.y;
	}

	avgParticle.x = avgParticle.x/particles.size();
	avgParticle.y = avgParticle.y/particles.size();
	
	for (int i = 0; i < particles.size(); i++){
		particles[i].resetForce();
		
		particles[i].addAttractionForce(avgParticle.x, avgParticle.y, 1500, 1.0);
		
		// get the force from the vector field: 
		ofxVec2f frc;
		frc = VF.getForceFromPos(particles[i].pos.x, particles[i].pos.y);
		particles[i].addForce(frc.x, frc.y);
		
		if (bParticlesRepel == true){
			for (int j = 0; j < i; j++){
				particles[i].addRepulsionForce(particles[j], 10, 1.5);
			}
		}
		
		particles[i].addForce(0,0.9f);
		particles[i].addDampingForce();
		particles[i].update();
		
		if (particles[i].pos.y > ofGetHeight()){
			particles[i].pos.x = ofRandom(400,600);
			particles[i].pos.y = 0;

		}
	}
	
	
	if (bFade == true) VF.fadeField(0.99f);

}

//--------------------------------------------------------------
void testApp::draw(){
	ofBackground(255, 255, 255);
	ofEnableAlphaBlending();
//	ofSetColor(0,130,130, 10);
//	VF.draw();
	
	ofSetColor(0, 0, 255, 10);
	
	for (int i = 0; i < particles.size(); i++){
//		particles[i].draw();
		ofCircle(particles[i].pos.x, particles[i].pos.y, 10);
	}
	
	ofSetColor(255, 0, 0, 100);
	ofCircle(avgParticle.x, avgParticle.y, 25);
/*	
	ofSetColor(0,130,130, 200);
	ofRect(30,30,300,75);
	ofSetColor(255,255,255, 230);
	ofDrawBitmapString("space to clear\ntoggle fade 'f'\ntoggle repel 'r'", 50, 50);
	
*/	
	ofSetColor(255,255,130, 230);
	ofDisableAlphaBlending();
}

//--------------------------------------------------------------
void testApp::keyPressed  (int key){ 
	
	if (key == ' '){
		VF.clear();
	} else if (key == 'f'){
		bFade = !bFade;
	} else if (key == 'r'){
		bParticlesRepel = !bParticlesRepel;
	}
}	

//--------------------------------------------------------------
void testApp::keyReleased  (int key){ 
}

//--------------------------------------------------------------
void testApp::mouseMoved(int x, int y ){
}

//--------------------------------------------------------------
void testApp::mouseDragged(int x, int y, int button){
	if (button == 0) {
		VF.addOutwardCircle((float)x, (float)y, 100, 0.29f);
	} else {
		
		particles.erase(particles.begin());
		particle myParticle;
		myParticle.setInitialCondition(x,y,0,0);
		particles.push_back(myParticle);
		
		
	}
}

//--------------------------------------------------------------
void testApp::mousePressed(int x, int y, int button){
}

//--------------------------------------------------------------
void testApp::mouseReleased(){
}
